/* $Id: saudio.c,v 3.3 1993/06/28 20:54:17 bjoerns Exp $
 *
 *	This file is part of the XPilot project, written by
 *
 *	    Bjrn Stabell (bjoerns@staff.cs.uit.no)
 *	    Ken Ronny Schouten (kenrsc@stud.cs.uit.no)
 *	    Bert Gsbers (bert@mc.bio.uva.nl)
 *
 *	Copylefts are explained in the LICENSE file.
 *
 *	This piece of code was provided by Greg Renda (greg@ncd.com).
 */

#include "global.h"

#define SOUND_RANGE_FACTOR	0.5		/* factor to increase sound
						 * range by */
#define SOUND_DEFAULT_RANGE	CENTER		/* from draw.h (384) */
#define SOUND_MAX_VOLUME	100
#define SOUND_MIN_VOLUME	10

#define sound_range(pl) \
	(SOUND_DEFAULT_RANGE + pl->sensors\
	 * SOUND_DEFAULT_RANGE * SOUND_RANGE_FACTOR)

typedef struct _AudioQRec
{
    int             index,
                    volume;
    struct _AudioQRec *next;
} AudioQRec, *AudioQPtr;


static void queue_audio(player * pl, int index, int volume)
{
    AudioQPtr       a, p, prev;

    if (!(a = (AudioQPtr) malloc(sizeof(AudioQRec))))
	return;

    a->index = index;
    a->volume = volume;
    a->next = NULL;

    p = prev = (AudioQPtr)pl->audio;

    while (p) {
	prev = p;
	p = p->next;
    }

    if (prev)
	prev->next = a;
    else
	pl->audio = (void *) a;
}

int sound_player_init(player * pl)
{
    return 0;
}

/*
 * Play a sound for a player.
 */
void sound_play_player(player * pl, int index)
{
    if (pl->conn != NOT_CONNECTED)
	queue_audio(pl, index, 100);
}

/*
 * Play a sound for all players.
 */
void sound_play_all(int index)
{
    int i;

    for (i = 0; i < NumPlayers; i++)
	sound_play_player(Players[i], index);
}

/*
 * Play a sound if location is within player's sound range. A player's sound
 * range depends on the number of sensors they have. The default sound range
 * is what the player can see on the screen. A volume is assigned to the
 * sound depending on the location within the sound range.
 */
void sound_play_sensors(float x, float y, int index)
{
    int             i,
                    volume;
    float           dx,
                    dy,
                    range,
                    factor;
    player         *pl;

    for (i = 0; i < NumPlayers; i++) {
	pl = Players[i];

	if (pl->conn == NOT_CONNECTED)
	    continue;

	dx = ABS(pl->pos.x - x);
	dy = ABS(pl->pos.y - y);
	range = sound_range(pl);

	if (dx >= 0 && dx <= range && dy >= 0 && dy <= range) {
	    /*
	     * scale the volume
	     */
	    factor = MAX(dx, dy) / range;
	    volume = MAX(SOUND_MAX_VOLUME - SOUND_MAX_VOLUME * factor,
			 SOUND_MIN_VOLUME);
	    queue_audio(pl, index, volume);
	}
    }
}

void sound_play_queued(player * pl)
{
    AudioQPtr       p,
                    n;

    p = (AudioQPtr)pl->audio;

    while (p) {
	n = p->next;
	Send_audio(pl->conn, p->index, p->volume);
	free(p);
	p = n;
    }

    pl->audio = NULL;
}

void sound_close(player * pl)
{
    AudioQPtr       p,
                    n;

    p = (AudioQPtr)pl->audio;

    while (p) {
	n = p->next;
	free(p);
	p = n;
    }

    pl->audio = NULL;
}
